@@ -10,7 +10,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #define YYPREFIX "yy"
 #line 2 "yacc_config.y"
 /*
- * yacc_config.y 1.43 1998/08/03 17:14:06
+ * yacc_config.y 1.44 1999/05/14 15:58:04
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -361,7 +361,7 @@ void yyerror(char *msg, ...)
 static int add_binding(card_info_t *card, char *name, int fn)
 {
     device_info_t *dev = root_device;
-    if (card->functions == MAX_FUNCTIONS) {
+    if (card->bindings == MAX_BINDINGS) {
 	yyerror("too many bindings\n");
 	return -1;
     }
@@ -371,9 +371,9 @@ static int add_binding(card_info_t *card, char *name, int fn)
 	yyerror("unknown device: %s", name);
 	return -1;
     }
-    card->device[card->functions] = dev;
-    card->dev_fn[card->functions] = fn;
-    card->functions++;
+    card->device[card->bindings] = dev;
+    card->dev_fn[card->bindings] = fn;
+    card->bindings++;
     free(name);
     return 0;
 }
@@ -581,7 +581,7 @@ case 5:
 			yyerror("no ID method for this card");
 			YYERROR;
 		    }
-		    if (yyvsp[0].card->functions == 0) {
+		    if (yyvsp[0].card->bindings == 0) {
 			yyerror("no function bindings");
 			YYERROR;
 		    }
