@@ -5,7 +5,7 @@
     This driver implements a disk-like block device driver with an
     apparent block size of 512 bytes for flash memory cards.
 
-    ftl_cs.c 1.39 1998/09/06 23:07:02
+    ftl_cs.c 1.42 1999/05/01 04:11:33
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.0 (the "License"); you may not use this file
@@ -88,7 +88,7 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ftl_cs.c 1.39 1998/09/06 23:07:02 (David Hinds)";
+"ftl_cs.c 1.42 1999/05/01 04:11:33 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -164,7 +164,7 @@ typedef struct partition_t {
 typedef struct ftl_dev_t {
     eraseq_handle_t	eraseq_handle;
     eraseq_entry_t	eraseq[MAX_ERASE];
-    struct wait_queue	*erase_pending;
+    wait_queue_head_t	erase_pending;
     partition_t		minor[CISTPL_MAX_DEVICES];
 } ftl_dev_t;
 
@@ -175,7 +175,7 @@ static struct hd_struct ftl_hd[MINOR_NR(MAX_DEV, 0, 0)];
 static int ftl_sizes[MINOR_NR(MAX_DEV, 0, 0)];
 static int ftl_blocksizes[MINOR_NR(MAX_DEV, 0, 0)];
 
-static struct wait_queue *ftl_wait_open = NULL;
+static wait_queue_head_t ftl_wait_open;
 
 static struct gendisk ftl_gendisk = {
     0,			/* Major device number */
@@ -259,7 +259,7 @@ static dev_link_t *ftl_attach(void)
     
     dev = kmalloc(sizeof(struct ftl_dev_t), GFP_KERNEL);
     memset(dev, 0, sizeof(ftl_dev_t));
-    init_waitqueue(&dev->erase_pending);
+    init_waitqueue_head(&dev->erase_pending);
     link->priv = dev;
 
     /* Register with Card Services */
@@ -818,6 +818,7 @@ static int copy_erase_unit(partition_t *part, u_short srcunit,
 
     /* Write the BAM to the transfer unit */
     req.Offset = xfer->Offset + part->header.BAMOffset;
+    req.Count = part->BlocksPerUnit * sizeof(int);
     ret = CardServices(WriteMemory, part->handle, &req, part->bam_cache);
     if (ret != CS_SUCCESS) goto write_error;
     
@@ -1064,6 +1065,7 @@ static int ftl_open(struct inode *inode, struct file *file)
 #endif
 	} else {
 	    CardServices(CloseMemory, partition->handle);
+	    partition->handle = NULL;
 	    printk(KERN_NOTICE "ftl_cs: FTL partition is invalid.\n");
 	    return -ENODEV;
 	}
@@ -1418,7 +1420,7 @@ static int ftl_reread_partitions(int minor)
 	    part->header.FormattedSize/SECTOR_SIZE;
     else
 	ftl_hd[whole].start_sect = -1;
-    resetup_one_dev(&ftl_gendisk, whole);
+    resetup_one_dev(&ftl_gendisk, whole >> PART_BITS);
     
 #ifdef PCMCIA_DEBUG
     for (i = 0; i < MAX_PART; i++) {
@@ -1516,7 +1518,7 @@ int init_module(void)
     blk_dev[major_dev].request_fn = DEVICE_REQUEST;
     ftl_gendisk.next = gendisk_head;
     gendisk_head = &ftl_gendisk;
-    init_waitqueue(&ftl_wait_open);
+    init_waitqueue_head(&ftl_wait_open);
     
     return 0;
 }
