@@ -1,5 +1,5 @@
 /*
- * ciscode.h 1.26 1998/05/14 00:13:11
+ * ciscode.h 1.32 1999/02/26 07:20:21
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -23,8 +23,12 @@
 
 #define MANFID_3COM			0x0101
 #define PRODID_3COM_3CXEM556		0x0035
+#define PRODID_3COM_3CCFEM556		0x0556
 #define PRODID_3COM_3C562		0x0562
 
+#define MANFID_ACCTON			0x01bf
+#define PRODID_ACCTON_EN2226		0x010a
+
 #define MANFID_ADAPTEC			0x012f
 #define PRODID_ADAPTEC_SCSI		0x0001
 
@@ -60,6 +64,10 @@
 
 #define MANFID_NEW_MEDIA		0x0057
 
+#define MANFID_OLICOM			0x0121
+#define PRODID_OLICOM_OC2231		0x3122
+#define PRODID_OLICOM_OC2232		0x3222
+
 #define MANFID_OMEGA			0x0137
 #define PRODID_OMEGA_QSP_100		0x0025
 
@@ -68,6 +76,8 @@
 #define PRODID_OSITECH_JACK_288		0x0002
 #define PRODID_OSITECH_JACK_336		0x0007
 
+#define MANFID_PSION			0x016c
+
 #define MANFID_QUATECH			0x0137
 #define PRODID_QUATECH_DUAL_RS232	0x0012
 #define PRODID_QUATECH_DUAL_RS232_D1	0x0007
@@ -78,6 +88,7 @@
 
 #define MANFID_SOCKET			0x0104
 #define PRODID_SOCKET_DUAL_RS232	0x0006
+#define PRODID_SOCKET_LPE		0x000d
 
 #define MANFID_SUNDISK			0x0045
 
