@@ -1,5 +1,5 @@
 /*
- * cs.h 1.60 1998/08/03 19:12:23
+ * cs.h 1.64 1999/04/09 03:21:28
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -87,6 +87,7 @@ typedef struct event_callback_args_t {
     void	*buffer;
     void	*misc;
     void	*client_data;
+    struct bus_operations *bus;
 } event_callback_args_t;
 
 /* for GetConfigurationInfo */
@@ -280,6 +281,7 @@ typedef struct win_req_t {
 #define WIN_SHARED		0x0040
 #define WIN_FIRST_SHARED	0x0080
 #define WIN_USE_WAIT		0x0100
+#define WIN_MAP_BELOW_1MB	0x0400
 #define WIN_PREFETCH		0x0800
 #define WIN_CACHEABLE		0x1000
 #define WIN_BAR_MASK		0xe000
@@ -426,7 +428,9 @@ extern int CardServices(int func, ...);
 #endif
 
 #ifdef __BEOS__
-#define CS_CLIENT_MODULE_NAME "bus_managers/pcmcia_cs/client/v1"
+#define SS_MODULE_NAME(s)	("busses/pcmcia/" s "/v1")
+#define MTD_MODULE_NAME(s)	("busses/pcmcia/" s "/v1")
+#define CS_CLIENT_MODULE_NAME	"bus_managers/pcmcia_cs/client/v1"
 typedef struct cs_client_module_info {
     bus_manager_info	binfo;
     int (*_CardServices)(int, ...);
@@ -441,6 +445,9 @@ typedef struct cs_socket_module_info {
     void (*_unregister_ss_entry)(ss_entry_t);
     void (*_add_timer)(struct timer_list *);
     void (*_del_timer)(struct timer_list *);
+    int (*register_resource)(int, u_long, u_long);
+    int (*release_resource)(int, u_long, u_long);
+    int (*check_resource)(int, u_long, u_long);
 } cs_socket_module_info;
 #endif
 
