@@ -89,7 +89,7 @@ typedef union ds_ioctl_arg_t {
 typedef struct dev_link_t {
     dev_node_t		*dev;
     u_int		state, open;
-    struct wait_queue	*pending;
+    wait_queue_head_t	pending;
     struct timer_list	release;
     client_handle_t	handle;
     io_req_t		io;
